import { __decorate, __assign } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Input, Output, Component, NgModule } from '@angular/core';
import Quagga from '@ericblade/quagga2';
import { Subject } from 'rxjs';

var NgxBarcodeScannerService = /** @class */ (function () {
    function NgxBarcodeScannerService() {
    }
    NgxBarcodeScannerService.prototype.defaultConfig = function () {
        return {
            inputStream: {
                name: 'Live',
                type: 'LiveStream',
            },
            locator: {
                patchSize: 'medium',
                halfSample: false
            },
            locate: true,
            numOfWorkers: 8,
            frequency: 10
        };
    };
    NgxBarcodeScannerService.prototype.meanBy = function (arr, property) {
        if (!arr) {
            return undefined;
        }
        return arr.reduce(function (acc, item) { return (property in item) ? acc + item[property] : acc; }, 0) / arr.length;
    };
    NgxBarcodeScannerService.prototype.isScanMatch = function (scanResult, errorThresholdPercentage) {
        var avgErrors = this.meanBy(scanResult.codeResult.decodedCodes, 'error');
        return avgErrors < errorThresholdPercentage;
    };
    NgxBarcodeScannerService.prototype.start = function (config, errorThresholdPercentage) {
        var _this = this;
        if (typeof this.scanResult === 'undefined') {
            this.scanResult = new Subject();
        }
        Quagga.onProcessed(function (scanResult) {
            _this.onProcessed(scanResult);
        });
        Quagga.onDetected(function (result) {
            var barcode = result.codeResult.code;
            if (_this.isScanMatch(result, errorThresholdPercentage)) {
                _this.scanResult.next(barcode + '');
            }
        });
        Quagga.init(config, function (error) {
            if (error) {
                _this.scanResult.error(error);
                _this.stop();
            }
            else {
                Quagga.start();
            }
        });
        return this.scanResult;
    };
    NgxBarcodeScannerService.prototype.stop = function () {
        if (typeof this.scanResult !== 'undefined') {
            this.scanResult.unsubscribe();
        }
        Quagga.stop();
    };
    NgxBarcodeScannerService.prototype.onProcessed = function (result) {
        var drawingCtx = Quagga.canvas.ctx.overlay;
        var drawingCanvas = Quagga.canvas.dom.overlay;
        if (result) {
            if (result.boxes) {
                drawingCtx.clearRect(0, 0, parseInt(drawingCanvas.getAttribute('width'), 10), parseInt(drawingCanvas.getAttribute('height'), 10));
                result.boxes.filter(function (box) { return box !== result.box; }).forEach(function (box) {
                    Quagga.ImageDebug.drawPath(box, { x: 0, y: 1 }, drawingCtx, { color: 'green', lineWidth: 2 });
                });
            }
            if (result.box) {
                Quagga.ImageDebug.drawPath(result.box, { x: 0, y: 1 }, drawingCtx, { color: '#00F', lineWidth: 2 });
            }
            if (result.codeResult && result.codeResult.code) {
                Quagga.ImageDebug.drawPath(result.line, { x: 'x', y: 'y' }, drawingCtx, { color: 'red', lineWidth: 3 });
            }
        }
    };
    NgxBarcodeScannerService.ɵprov = ɵɵdefineInjectable({ factory: function NgxBarcodeScannerService_Factory() { return new NgxBarcodeScannerService(); }, token: NgxBarcodeScannerService, providedIn: "root" });
    NgxBarcodeScannerService = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], NgxBarcodeScannerService);
    return NgxBarcodeScannerService;
}());

var Utils = /** @class */ (function () {
    function Utils() {
    }
    Utils.setOrDefault = function (object, path, value) {
        if (typeof object[path] === 'undefined') {
            object[path] = value;
        }
    };
    return Utils;
}());

var NgxBarcodeScannerComponent = /** @class */ (function () {
    function NgxBarcodeScannerComponent(service) {
        this.service = service;
        this.codes = [
            'code_128', 'ean', 'ean_8', 'code_39', 'code_39_vin',
            'codabar', 'upc', 'upc_e', 'i2of5', '2of5', 'code_93'
        ];
        this.valueChange = new EventEmitter();
        this.exception = new EventEmitter();
    }
    NgxBarcodeScannerComponent.prototype.setConfig = function () {
        if (!this.config) {
            this.config = __assign(__assign({}, this.service.defaultConfig()), { decoder: {
                    readers: this.readers()
                } });
        }
        if (!this.config.inputStream) {
            this.config.inputStream = {};
        }
        Utils.setOrDefault(this.config.inputStream, 'name', 'Live');
        Utils.setOrDefault(this.config.inputStream, 'type', 'LiveStream');
        if (!this.config.locator) {
            this.config.locator = {};
        }
        Utils.setOrDefault(this.config.locator, 'patchSize', 'medium');
        Utils.setOrDefault(this.config.locator, 'halfSample', false);
        Utils.setOrDefault(this.config, 'locate', true);
        Utils.setOrDefault(this.config, 'numOfWorkers', 8);
        Utils.setOrDefault(this.config, 'frequency', 10);
        if (!this.config.decoder) {
            this.config.decoder = {};
        }
        Utils.setOrDefault(this.config.decoder, 'readers', this.readers());
    };
    NgxBarcodeScannerComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.setConfig();
        var threshold = isNaN(this.errorThreshold) ? 0.1 : this.errorThreshold;
        this.subscription = this.service.start(this.config, threshold).subscribe(function (value) {
            _this.valueChange.emit(value);
            _this.service.stop();
        }, function (error) {
            _this.exception.emit(error);
        });
    };
    NgxBarcodeScannerComponent.prototype.ngOnDestroy = function () {
        this.service.stop();
        if (this.subscription)
            this.subscription.unsubscribe();
    };
    NgxBarcodeScannerComponent.prototype.readers = function () {
        var types = (typeof this.codes === 'string') ? [this.codes] : this.codes;
        return types.map(function (it) { return it + '_reader'; });
    };
    NgxBarcodeScannerComponent.ctorParameters = function () { return [
        { type: NgxBarcodeScannerService }
    ]; };
    __decorate([
        Input()
    ], NgxBarcodeScannerComponent.prototype, "codes", void 0);
    __decorate([
        Input()
    ], NgxBarcodeScannerComponent.prototype, "config", void 0);
    __decorate([
        Input()
    ], NgxBarcodeScannerComponent.prototype, "errorThreshold", void 0);
    __decorate([
        Input()
    ], NgxBarcodeScannerComponent.prototype, "value", void 0);
    __decorate([
        Output()
    ], NgxBarcodeScannerComponent.prototype, "valueChange", void 0);
    __decorate([
        Output()
    ], NgxBarcodeScannerComponent.prototype, "exception", void 0);
    NgxBarcodeScannerComponent = __decorate([
        Component({
            // tslint:disable-next-line:component-selector
            selector: 'ngx-barcode-scanner',
            template: "<div id=\"interactive\" class=\"viewport\"></div>\n",
            styles: ["::ng-deep #interactive.viewport{position:relative;width:100%;height:auto;overflow:hidden;text-align:center}::ng-deep #interactive.viewport>canvas,::ng-deep #interactive.viewport>video{max-width:100%;width:100%}::ng-deep canvas.drawing,::ng-deep canvas.drawingBuffer{position:absolute;left:0;top:0}::ng-deep video{width:100%;-o-object-fit:cover;object-fit:cover}:host{position:static;width:100%}"]
        })
    ], NgxBarcodeScannerComponent);
    return NgxBarcodeScannerComponent;
}());

var NgxBarcodeScannerModule = /** @class */ (function () {
    function NgxBarcodeScannerModule() {
    }
    NgxBarcodeScannerModule = __decorate([
        NgModule({
            declarations: [NgxBarcodeScannerComponent],
            imports: [],
            exports: [NgxBarcodeScannerComponent]
        })
    ], NgxBarcodeScannerModule);
    return NgxBarcodeScannerModule;
}());

/*
 * Public API Surface of ngx-barcode-scanner
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxBarcodeScannerComponent, NgxBarcodeScannerModule, NgxBarcodeScannerService };
//# sourceMappingURL=baivoom-ngx-barcode-scanner.js.map
